[PATCH v2 2/3] rust: sync: Introduce lock::Backend::Context

From: Lyude Paul
Date: Wed Jul 31 2024 - 18:41:25 EST


Now that we've introduced an `IrqDisabled` token for marking contexts in
which IRQs are disabled, we need a way to be able to pass it to locks that
require that IRQs are disabled. In order to continue using the
`lock::Backend` type instead of inventing our own thing, we accomplish this
by adding the associated Context type, along with a `lock_with()` function
that can accept a Context when acquiring a lock. To allow current users of
context-less locks to keep using the normal `lock()` method, we take an
example from Wedson Almeida Filho's work and add a `where T<'a>: Default`
bound to `lock()` so that it can only be called on lock types where the
context is simply a placeholder value, then re-implement it through the new
`lock_with()` function.

Signed-off-by: Lyude Paul <lyude@xxxxxxxxxx>
---
rust/kernel/sync/lock.rs | 17 +++++++++++++++--
rust/kernel/sync/lock/mutex.rs | 3 +++
rust/kernel/sync/lock/spinlock.rs | 1 +
3 files changed, 19 insertions(+), 2 deletions(-)

diff --git a/rust/kernel/sync/lock/mutex.rs b/rust/kernel/sync/lock/mutex.rs
index 30632070ee670..327e53be4c0f4 100644
--- a/rust/kernel/sync/lock/mutex.rs
+++ b/rust/kernel/sync/lock/mutex.rs
@@ -4,6 +4,8 @@
//!
//! This module allows Rust code to use the kernel's `struct mutex`.

+use core::marker::*;
+
/// Creates a [`Mutex`] initialiser with the given name and a newly-created lock class.
///
/// It uses the name if one is given, otherwise it generates one based on the file name and line
@@ -93,6 +95,7 @@ macro_rules! new_mutex {
unsafe impl super::Backend for MutexBackend {
type State = bindings::mutex;
type GuardState = ();
+ type Context<'a> = PhantomData<&'a ()>;

unsafe fn init(
ptr: *mut Self::State,
diff --git a/rust/kernel/sync/lock/spinlock.rs b/rust/kernel/sync/lock/spinlock.rs
index ea5c5bc1ce12e..8503d6e8e3de3 100644
--- a/rust/kernel/sync/lock/spinlock.rs
+++ b/rust/kernel/sync/lock/spinlock.rs
@@ -92,6 +92,7 @@ macro_rules! new_spinlock {
unsafe impl super::Backend for SpinLockBackend {
type State = bindings::spinlock_t;
type GuardState = ();
+ type Context<'a> = PhantomData<&'a ()>;

unsafe fn init(
ptr: *mut Self::State,
diff --git a/rust/kernel/sync/lock.rs b/rust/kernel/sync/lock.rs
index f6c34ca4d819f..f3ff50c4272cf 100644
--- a/rust/kernel/sync/lock.rs
+++ b/rust/kernel/sync/lock.rs
@@ -38,6 +38,9 @@ pub unsafe trait Backend {
/// [`unlock`]: Backend::unlock
type GuardState;

+ /// The context which much be provided to acquire the lock.
+ type Context<'a>;
+
/// Initialises the lock.
///
/// # Safety
@@ -120,14 +123,24 @@ pub fn new(t: T, name: &'static CStr, key: &'static LockClassKey) -> impl PinIni
}

impl<T: ?Sized, B: Backend> Lock<T, B> {
- /// Acquires the lock and gives the caller access to the data protected by it.
- pub fn lock(&self) -> Guard<'_, T, B> {
+ /// Acquires the lock with the given context and gives the caller access to the data protected
+ /// by it.
+ pub fn lock_with(&self, _context: B::Context<'_>) -> Guard<'_, T, B> {
// SAFETY: The constructor of the type calls `init`, so the existence of the object proves
// that `init` was called.
let state = unsafe { B::lock(self.state.get()) };
// SAFETY: The lock was just acquired.
unsafe { Guard::new(self, state) }
}
+
+ /// Acquires the lock and gives the caller access to the data protected by it.
+ #[inline]
+ pub fn lock<'a>(&'a self) -> Guard<'a, T, B>
+ where
+ B::Context<'a>: Default,
+ {
+ self.lock_with(B::Context::default())
+ }
}

/// A lock guard.
--
2.45.2